#' Calculate integrals in the expansion of a bivariate cumulant generating function.
#' @param zeta Slope of the boundary of the region whose probability is desired.
#' @param omegahat Signed roots of the likelihood ratio statistics
#' @param n Number of independent summands
#' @param Psibar21 Conditional CDF of from bivariate normal, times marginal density.
#' @param psi12 Bivariate normal density.
#' @export
#' @return A list with various integrals
integs<-function(zeta,omegahat,n=1,Psibar21=NA,psi12=NA){
   if(n>1) omegahat<-sqrt(n)*omegahat
   if(is.na(Psibar21)) 
      Psibar21<-(dnorm((omegahat[1] + zeta*omegahat[2])/sqrt(1 + zeta^ 2))* 
         pnorm(-sqrt(1/(1 + zeta^ 2))*(-(zeta*omegahat[1]) + omegahat[2])))/ sqrt(1 + zeta^ 2)
   if(is.na(psi12)) 
      psi12<-sqrt(1/(1 + zeta^ 2))*sqrt(1 + zeta^ 2)* dnorm(sqrt(1/(1 + zeta^ 2))*
         (-(zeta*omegahat[1]) + omegahat[2]))* dnorm((omegahat[1] + zeta*omegahat[2])/sqrt(1 + zeta^ 2))
   k0m00<- Psibar21
   k0m11<- -((zeta*(zeta*omegahat[1] - omegahat[2])*
      (psi12 + Psibar21*zeta*omegahat[1] - Psibar21*omegahat[2]))/(1 + zeta^ 2))
   k0m20<- (psi12*zeta^ 2*(zeta*omegahat[1] - omegahat[2]) + 
      Psibar21*(-1 + zeta^ 4*omegahat[1]^ 2 - 2*zeta^ 3*omegahat[1]*omegahat[2] + 
      zeta^ 2*(-1 + omegahat[2]^ 2)))/(1 + zeta^ 2)^ 2
   return(c( k0m00= k0m00, k0m11= k0m11, k0m20= k0m20))
}
