#optim(c(0,0),function(tauv,xb) return(-(tauv[1]*xb[1]+tauv[2]*xb[2]-
#    log(1-tauv[1])-log(1-tauv[2])-log(1-tauv[1]-tauv[2]))),xb=c(1,2))
#a <- function(tauv,xb){
#   cat("Argument tauv to a",tauv,"\n")
#   if((sum(tauv)<1)&&(tauv[1]<1)&& (tauv[2]<1)){
#      d1 <- -c(((2*tauv[1] + tauv[2]-2)/((tauv[1]-1)*(sum(tauv)-1))),
#              ((tauv[1] + 2*tauv[2]-2)/((tauv[2]-1)*(sum(tauv)-1))))
#   }else{
#      d1 <- rep(NA,2)
#   }
#   return(d1-xb)
#}
#b <- function(tau2,x2){ cat("Entering function b\n");
#   return(a(c(0,tau2),xb=c(0,x2))[2])}
#nleqslv(c(0,0),a,xb=c(1,1))
#nleqslv(0,b,x2=1)
#' Cumulant generating function for two independent exponentials
#' @param tauv vector with two components; argument to multivariate cgf.
#' @return a list with comonents d0, a scalar representing the cgf argument, d1, a vector with two components representing the cgf derivative, and d2, a 2x2 matrix of second derivatives.
#' @export
Kindexp <- function(tauv){
# K[tau1_,tau2_]=-Log[1-tau1]-Log[1-tau2]
    if((tauv[1]<1)&& (tauv[2]<1)){
       d0 <- -sum(log(1-tauv))
       d1 <- 1/(1-tauv)
       d2 <- diag(1/(1-tauv)^2)
   }else{
      d0 <- NA
      d1 <- c(NA,NA)
      d2 <- array(NA,c(2,2))
   }
#  cat("Leaving Kindexp, d1",d1,"\n")
   return(list(d0=d0,d1=d1,d2=d2))
}
#' Cumulant generating function for sums of independent exponentials, with one variable shared between the two components.
#' @param tauv vector with two components; argument to multivariate cgf.
#' @export
#' @return a list with comonents d0, a scalar representing the cgf argument, d1, a vector with two components representing the cgf derivative, and d2, a 2x2 matrix of second derivatives.
Kdep <- function(tauv){
# K[tau1_,tau2_]=-Log[1-tau1]-Log[1-tau2]-Log[1-tau1-tau2]
# D[D[D[K[tau1,tau2],tau2],tau2],tau2]
#  if(sum(tauv)>=1) cat("Problem\n")
#  cat("Entered Kdep, tauv\n"); print(tauv)
   if((sum(tauv)<1)&&(tauv[1]<1)&& (tauv[2]<1)){
      d0 <- -sum(log(1-tauv))-log(1-sum(tauv))
      d1 <- 1/(1-tauv)+ 1/(1-sum(tauv))
      d2 <- diag(1/(1-tauv)^2)+1/(1-sum(tauv))^2
      d3<-array(0,c(2,2,2))+2/(1-sum(tauv))^3
      d3[1,1,1]<-d3[1,1,1]+2/(1-tauv[1])^3
      d3[2,2,2]<-d3[2,2,2]+2/(1-tauv[2])^3
   }else{
      d0 <- NA
      d1 <- c(NA,NA)
      d2 <- array(NA,c(2,2))
      d3 <- array(NA,c(2,2,2))
   }
#  cat("Leaving Kdep, d1",d1,"\n")
   return(list(d0=d0,d1=d1,d2=d2,d3=d3))
}
#' Cumulant generating function for two independent normals
#' @param tauv vector with two components; argument to multivariate cgf.
#' @return a list with comonents d0, a scalar representing the cgf argument, d1, a vector with two components representing the cgf derivative, and d2, a 2x2 matrix of second derivatives.
#' @export
Kindnorm <- function(tauv){
#  cat("Entering Kindnorm tauv",tauv,"\n")
   d0 <- sum(tauv^2)/2
   d1 <- tauv
   d2 <- diag(c(1,1))
#  cat("Kindnorm d0=",d0,"\n")
   return(list(d0=d0,d1=d1,d2=d2))
}
#' Cumulant generating function for a bivariate correlated normal distribution.
#' @param tauv vector with two components; argument to multivariate cgf.
#' @export
#' @return a list with comonents d0, a scalar representing the cgf argument, d1, a vector with two components representing the cgf derivative, and d2, a 2x2 matrix of second derivatives.
Kindnor <- function(tauv){
#  cat("Entering Kbvn tauv",tauv,"\n")
#K[tau1_,tau2_]=tau1^2/2+tau2^2/2-tau1 tau2/2
#Table[D[D[K[tau1,tau2],{tau1,tau2}[[i]] ],{tau1,tau2}[[j]]],{i,1,2},{j,1,2}]
   d0 <- sum(tauv^2)/2
   d1 <- tauv
   d2 <- array(c(1,0,0,1),c(2,2))
#  cat("Kbvn d0=",d0,"\n")
   return(list(d0=d0,d1=d1,d2=d2))
}
#' Cumulant generating function for a bivariate correlated normal distribution.
#' @param tauv vector with two components; argument to multivariate cgf.
#' @export
#' @return a list with comonents d0, a scalar representing the cgf argument, d1, a vector with two components representing the cgf derivative, and d2, a 2x2 matrix of second derivatives.
Kbvn <- function(tauv){
#  cat("Entering Kbvn tauv",tauv,"\n")
#K[tau1_,tau2_]=tau1^2/2+tau2^2/2-tau1 tau2/2
#Table[D[D[K[tau1,tau2],{tau1,tau2}[[i]] ],{tau1,tau2}[[j]]],{i,1,2},{j,1,2}]
   d0 <- sum(tauv^2)/2-tauv[1]*tauv[2]/2
   d1 <- c(tauv[1]-tauv[2]/2, tauv[2]-tauv[1]/2)
   d2 <- array(c(1,-1/2,-1/2,1),c(2,2))
#  cat("Kbvn d0=",d0,"\n")
   return(list(d0=d0,d1=d1,d2=d2))
}
#' Simulates sample of size mcsize representing averages of n exponential variables
#' @param mcsize Number of Monte Carlo samples.
#' @param n Number of components in average to be simulated.
#' @return A matrix with two columns and mcsize rows, containing the samples.
#' @export
#' @importFrom stats rgamma
simindexp<-function(mcsize,n){
   v2 <- rgamma(mcsize,n)/n
   v3 <- rgamma(mcsize,n)/n
   return(cbind(v2,v3))
}
#' Simulates sample of size mcsize representing averages of sums of n sets of two exponential variables, with one shared between the two components.
#' @param mcsize Number of Monte Carlo samples.
#' @param n Number of components in average to be simulated.
#' @return A matrix with two columns and mcsize rows, containing the samples.
#' @export
simgamma<-function(mcsize,n){
   v1 <- rgamma(mcsize,n)/n
   v2 <- rgamma(mcsize,n)/n
   v3 <- rgamma(mcsize,n)/n
   return(cbind(v1+v2,v1+v3))
}
#' Simulates sample of size mcsize representing independent normal variables.
#' @param mcsize Number of Monte Carlo samples.
#' @param n Number of components in average to be simulated.  Not used.
#' @return A matrix with two columns and mcsize rows, containing the samples.
#' @importFrom stats rnorm cor
#' @export
simindnor<-function(mcsize,n){
   v1 <- rnorm(mcsize)
   v2 <- rnorm(mcsize)
   return(cbind(v1,v2))
}
#' Simulates sample of size mcsize representing correlated bivariate normal variables.
#' @param mcsize Number of Monte Carlo samples.
#' @param n Number of components in average to be simulated.  Not used.
#' @return A matrix with two columns and mcsize rows, containing the samples.
#' @importFrom stats rnorm cor
#' @export
simbvn<-function(mcsize,n){
   v1 <- sqrt(.5)*rnorm(mcsize)
   v2 <- sqrt(.5)*rnorm(mcsize)
   v3 <- sqrt(.5)*rnorm(mcsize)
   return(cbind(v1+v3,-v1-v2))
}
#wvec(array(0,c(1,2)),c(4,4))
#aa<- myplot()
#hatwvec(xvec=c(4,4))
#out<-rep(NA,101) ; for(jj in seq(length(out))){hv<-hatwvec(xvec=c(3,3.5+(jj-1)/100),K=Kdep); out[jj]<-hv$what[1]}
#hv<-hatwvec(xvec=c(3,4),K=Kdep)
#delta<-0.01
