#' Value of first arguement of CGF corresponding to a value of first SRLLR parameter.
#' @param w1 first SRLLR parameter.
#' @param xvec Bivariate data vector
#' @param K Cumulant generating function
#' @param hv Output from hatwvec associated with xvec and K.
#' @return Value of first arguement of CGF corresponding to a w1.
#' @export
#' @importFrom nleqslv nleqslv
tau1f<-function(w1,xvec,K,hv){
#  cat("Entering tau1f w1=",w1,"xvec=",xvec,"\n")
   newf<-function(tauv,xvec,K,hv,w1){
      out<-2*(K(tauv)$d0 -sum(tauv*xvec)+hv$big)
      if(!is.na(out)) if(out<0){
         if(out<1.0e-10) cat("tauv",tauv,"out",out,"\n") else out<-0
      }
      outa<-c(hv$what[1]+sign(tauv[1]-hv$hattau[1])*sqrt(out)-w1,K(tauv)$d1[2]-xvec[2])
#     cat("In newf hattau[1]",hv$hattau[1],"tauv[1]=",tauv[1],"out",out,"outa",outa,"\n")
      return(outa)
   }
   jj<-function(tauv,xvec,K,hv,w1){
      Kout<-K(tauv)
      bot<-sqrt(2*(Kout$d0-sum(xvec*tauv)+hv$big))
      M<-array(c((Kout$d1[1]-xvec[1])/bot,Kout$d2[2,2]-xvec[1],
         (Kout$d1[2]-xvec[2])/bot,Kout$d2[1,2]),c(2,2))
      return(M)
   }
   start<-c(w1,0)
   count<-0
   while(count<6){
      if(is.na(K(start)$d0)){
         count<-count+1
         start<-start/2
      }else{
         count<-6
      }
   }
   out1<-nleqslv(start,newf,jac=NULL,xvec,K,hv,w1,jacobian=TRUE,control=list(xtol=1.0e-12))
#  print(out1$jac)
#  print(jj(out1$x,xvec,K,hv,w1))
#  out2<-nleqslv(c(0,0),newf,jac=jj,xvec,K,hv,w1,control=list(xtol=1.0e-12))
#  cat("tau1f termcd",out1$termcd,"out1$x[1]",out1$x[1],"\n")
   return(out1$x[1])
}
