#' Test correct calculation of various quantities involved in bivariate saddlepint tail probability approximation.
#' @param tv ordinate vector with two components
#' @param sigmav standard deviation vector with two components
#' @param rho correlation
#' @param omegahat Fitted values of omega
#' @param zeta2 Fitted slope
#' @return The naive correction to the bivariate normal approximation.
#' @importFrom stats dnorm pnorm
#' @export
testintegrals<-function(tv,sigmav=NULL,rho=NULL,omegahat=NULL,zeta2=NULL){
    if((!is.null(omegahat))|(!is.null(zeta2))){
       sigmav<-c(sqrt(1+zeta2^2),1)
       rho<-zeta2/sqrt(1+zeta2^2)
       tv<-c(omegahat[1]+zeta2*omegahat[2],omegahat[2])
    }
#   cat("tv",tv,"sigmav",sigmav,"rho",rho,"\n")
    dd<-dnorm(tv,sd=sigmav)
#   cat("In fun.testing dd",dd,"\n")
    dd<-dnorm(tv/sigmav)*(1/sigmav)
    pp<-pnorm(-(rev(tv)-rho*rev(sigmav)*tv/sigmav)/(rev(sigmav)*sqrt(1-rho^2)))
#   cat("dd",dd,"pp",pp,"\n")
    print(integs(zeta2,omegahat))
    return(c( dnorm(tv[1]/sigmav[1])*(1/sigmav[1])* pnorm(-(tv[2]-rho*sigmav[2]*tv[1]/sigmav[1])/(sigmav[2]*sqrt(1-rho^2))),
       dnorm(tv[2]/sigmav[2])*(1/sigmav[2])*
       pnorm(-(tv[1]-rho*sigmav[1]*tv[2]/sigmav[2])/(sigmav[1]*sqrt(1-rho^2))))
   )
}
