#' Test w1 and derivatives
#' @param xvec vector of means near which to calculate w
#' @param K cumulant generating function.
#' @param npts Number of points at which to evaluate
#' @param kk Component of w to calculate derivative of
#' @importFrom graphics lines legend
#' @export
#' @examples
#' testw(c(3,4.0),Kdep)
#' testw(c(3,4.5),Kdep)
testw <- function(xvec,K,npts=101,kk=1){
   mm <-1
   myK <- K
   out1 <- rep(NA,npts)
   out2 <- rep(NA,length(out1)-1)
   tau1v <- ((0:(length(out1)-1))-(length(out1)-1)/2)/100
   hv <- hatwvec(xvec=xvec,K=myK)
   for(jj in seq(length(out1))){
#     cat("jj",jj,"\n")
      tau1 <- tau1v[jj]
      out1[jj] <- wvec(c(tau1,0),xvec,hv,myK)[1,kk]
      tau1s <- tau1+0.005
#     temp <- if(kk==1) myK(c(tau1s,tautilde(tau1s,xvec,myK)))$d1[kk] else myK(c(tau1s,0))$d1[kk]
      if(jj>1){
         bot <- wvec(c(tau1s,0),xvec,hv,myK)[1,kk]-hv$what[kk]
#        cat("bot",bot,"\n")
         if(abs(bot)<1.0e-10){
            Khatd2 <- K(hv$hattau)$d2
            out2[jj-1] <- sqrt(Khatd2[1,1]-Khatd2[2,1]*Khatd2[1,2]/Khatd2[2,2])
         }else{
            taut <- tautilde(tau1s,xvec,myK)
            if(kk==1) top <- myK(c(tau1s,taut))$d1[mm]- xvec[mm]
            if(kk==2) top <- myK(c(tau1s,0))$d1[mm]- myK(c(tau1s,taut))$d1[mm]
#           cat("top",top,"\n")
            out2[jj-1] <- top/bot
         }
      }
   }
   ss=paste("xvec",paste(xvec,collapse=" "),paste("what",
         paste(round(hv$what,3),collapse=" ")))
   par(mfrow=c(2,2))
   midtau <- (tau1v[-1]+tau1v[-length(tau1v)])/2
   plot(tau1v,out1,sub=ss,xlab="tau1",ylab=paste("w",kk,sep=""),
      main=paste("Component ",kk))
   plot(midtau,diff(out1)/0.01, sub=ss,type="l",
      main=paste("Derivative of component ",kk, " re argument ",mm))
   lines(midtau,out2,lty=2)
#  cat("min(midtau)",min(midtau),"max(out2)",max(out2),"\n")
   legend(min(midtau),max(out2),lty=1:2,legend=c("Numerical","Analytic"))
   return(invisible(list(
      d0=list(tau=tau1v,w=out1),
      d1=list(tau=(tau1v[-1]+tau1v[-length(tau1v)])/2,numeric=diff(out1)/0.01,
         analytic=out2))))
}
