#' Calculate the two-dimensional LR saddlepoint approximation.  Also, sketch out the boundary of the region whose probability is calcualted.
#' @param xvec Two-component vector at which to calculate the tail probability.
#' @param n number of independent bivariate vectors making up the vector mean whose probability is being calculated.
#' @param K the bivariate cumulant generating function of each summand
#' @param stats A structure of test statistics.  Will be reconstructed if not input.
#' @param verbose Logical flag triggering verbose output.
#' @return A list consisting of a vector of various approximation values, and the slope.
#' @examples twodsp()
#' @export
#' @importFrom stats dnorm pnorm
#' @importFrom VGAM pbinorm
twodsp <- function(xvec=c(4,5),n=1, K=Kdep,stats=NULL,verbose=FALSE){
#  cat("Entered twodsp, xvec=",xvec,"\n")
   myK<-K
   if(is.null(stats)) stats <- hatwvec(xvec=xvec,K=myK,verbose=verbose)
   if(verbose) cat("n=",n,"\n")
   Khat<-K(stats$hattau)
# Added 3 Oct 2023
   Kcheck<-K(stats$tildetau)
   Kout<-K(c(0,0))
   if(verbose) cat("Kout$d1",Kout$d1,"\n")
# Remember zeta2=-slope[2]
   if(verbose) cat("stats$what",stats$what,"\n")
   if(verbose) cat("-stats$slope[2]",-stats$slope[2],"\n")
   lrnorm <- if(!any(is.na(stats$hattau)))
      pbinorm(-sqrt(n)*(stats$what[1]+stats$what[2]*(-stats$slope[2])),
         -sqrt(n)*stats$what[2],mean1=0,mean2=0,var1=1+stats$slope[2]^2, 
         var2=1,cov12=-stats$slope[2])
   else NA
#  cat("Covariance",-stats$slope[2],"\n")
   if(verbose) cat("stats$hattau",stats$hattau,"\n")
   if(verbose) cat("Normal approximation to bivariate distribution of SRLLR",lrnorm,"\n")
   sigmav<-sqrt(c(Khat$d2[1,1]-Khat$d2[1,2]*Khat$d2[2,1]/Khat$d2[2,2],Khat$d2[2,2]))
   zv<-stats$hattau*sigmav
# Change 3 Oct 2023
   sigmav<-sqrt(c(Khat$d2[1,1]-Khat$d2[1,2]*Khat$d2[2,1]/Khat$d2[2,2],Kcheck$d2[2,2]))
   zv<-c(stats$hattau[1],stats$tildetau[2])*sigmav
   hgv<-c(stats$what[1], stats$what[2]-(-stats$slope[2])*stats$what[1])
   hgv<-c(stats$what[1], stats$what[2])
   if(verbose) cat("sigmav",sigmav,"hat tau",stats$hattau,"zv",zv,"hgv",hgv,"\n")
   dd<-c(dnorm(sqrt(n)*(stats$what[1]+stats$what[2]*(-stats$slope[2]))/sqrt(1+stats$slope[2]^2))/sqrt(1+stats$slope[2]^2),dnorm(sqrt(n)*stats$what[2]))
   pp<-c(pnorm(-sqrt(n)*(stats$what[2]-(-stats$slope[2])* stats$what[1])/sqrt(1+stats$slope[2]^2)),
      pnorm(-sqrt(n)*stats$what[1]))
   if(verbose) cat("dd",dd,"pp",pp,"pp*dd",pp*dd,"fractions",(1/zv-1/hgv),"prods",dd*pp*(1/zv-1/hgv),"\n")
   diffv<-1/zv-1/hgv
# K[0,0]=0
# hatomega2[tildetwo_]=Sqrt[2(tildetwo*Derivative[0,1][K][0,tildetwo]-K[0,tildetwo])]
# Series[1/hatomega2[tildetwo],{tildetwo,0,0}]
   if(abs(stats$tildetau[1])<0.00001){
      diffv[1]<-(-((Khat$d2[1,2]*(Khat$d2[1,2]*Khat$d3[1,2,2] - 2*Khat$d2[2,2]*Khat$d3[1,1,2]))/Khat$d2[2,2]**2) - 
            Khat$d3[1,1,1] + ((Khat$d3[2,2,2]*Khat$d2[1,2]**3)/Khat$d2[2,2]**3 - 
              (3*Khat$d2[1,2]*Khat$d3[1,1,2])/Khat$d2[2,2] + 2*Khat$d3[1,1,1])/3.)/
         (2.*(-(Khat$d2[1,2]**2/Khat$d2[2,2]) + Khat$d2[1,1])**1.5)
   }
   if(abs(stats$tildetau[2])<0.00001){
      diffv[2]<--Kcheck$d3[2,2,2]/(6*Kcheck$d2[2,2]^1.5)
   }
   if(verbose) cat("diffv[2]",diffv[2],"xvec",xvec,"zv",zv,"hgv",hgv,"names(stats)",names(stats),"stats$tildetau[2]",stats$tildetau[2],"\n")
#  rawlr<- if(all(abs(hgv)>0.0001)) lrnorm+sum(pp*dd*diffv)/sqrt(n) else NA
   rawlr<- lrnorm+sum(pp*dd*diffv)/sqrt(n)
   if(verbose) cat("Bivariate SRLLR corrected a la LR",rawlr,"\n")
   clt<-pbinorm(-sqrt(n)*(xvec[1]-Kout$d1[1]),-sqrt(n)*(xvec[2]-Kout$d1[2]),mean1=0,mean2=0, 
      var1=Kout$d2[1,1], var2=Kout$d2[2,2], cov12=Kout$d2[1,2])
   integras<-integs(-stats$slope[2],sqrt(n)*stats$what)
   if(verbose) print(integras)
   zeta2 <- -stats$slope[2]
   zp <- stats$gp
   vars<-c("omegahat1","omegahat2","zeta2","zeta21")
   vals<-c(stats$what,zeta2,zp)
   cat("myrules={",paste(vars,vals,sep="->",collapse=","),"}\n",file="tempa.m")
   vals[4]<-0
   cat("myrules0={",paste(vars,vals,sep="->",collapse=","),"}\n",file="tempa.m",
      append=TRUE)
   if(verbose) cat("zeta2",zeta2,"zeta21",zp,"\n")
   c1 <- as.numeric(zp*integras["k0m11"])
   c2 <- as.numeric(zp*(zeta2+zp*hgv[1]/2)*integras["k0m20"])
   if(verbose) cat("c1",c1,"c2",c2,"\n")
   if(verbose) cat("zp",zp,"zp*(zeta2+zp*hgv[1]/2",zp*(zeta2+zp*hgv[1]/2),"\n")
   corr <- as.numeric(zp*(integras["k0m11"]+(zeta2+zp*hgv[1]/2)*integras["k0m20"]))/sqrt(n)
#  browser()
#  cat("corr",corr,"\n")
   final=rawlr+corr
   if(verbose) cat("Bivariate Saddlepoint including curve",final,"\n")
   return(list(approx=c(clt=clt,lrnorm=lrnorm,rawlr=rawlr,final=final),
      zeta2=zeta2,zeta21=zp,hattau=stats$hattau,hgv=hgv))
}
