#' Calculate a matrix of signed root of the likelihood ratio parameters.
#' @param taumat A matrix of cumulant generating argument parameters.
#' @param xvec A vector with two components, representing values of observation means.
#' @param hv A list with components representing statistics, including maximum likelihood estimates and signed root of the likelihood ratio statistics.
#' @param K A bivariate cumulant generating function
#' @return a matrix of the same shape as taumat, with rows representing separate parameter values, and two columns corresponding to components of the data vector.
#' @export
wvec <- function(taumat,xvec,hv,K){
#  cat("Entered wvec\n")
   myK<-K
   if(is.null(dim(taumat))&&(length(taumat)==2)) taumat <- array(taumat,c(1,2))
   out <- taumat
   qq <- rep(NA,3)
   eps <- 0.0000001
   for(jj in seq(dim(taumat)[1])){
      qq[1] <- -2*hv$big
#     cat("Before tautilde in wvec, inside loop\n")
      tt <- tautilde(taumat[jj,1],xvec,K=myK)
#     cat("xvec",xvec,"tt",tt, "hv$hattau", hv$hattau,"taumat[jj,]",taumat[jj,],"\n");#print(K)
#     cat("After tautilde in wvec, inside loop\n")
      qq[2] <- 2*(K(c(taumat[jj,1],tt))$d0-taumat[jj,1]*xvec[1]-tt*xvec[2])
      qq[3] <- 2*(K(taumat[jj,])$d0-sum(taumat[jj,]*xvec))
      if(qq[2]-qq[1]< -eps) cat("Error 1\n")
      if(qq[3]-qq[2]< -eps) cat("Error 2\n")
      out[jj,1] <- hv$what[1]+sign(taumat[jj,1]-hv$hattau[1])*sqrt(max(qq[2]-qq[1],0))
      out[jj,2] <- hv$what[2]+sign(taumat[jj,2]-tt)*sqrt(max(qq[3]-qq[2],0))
#     cat("sign(taumat[jj,2]-tt)",sign(taumat[jj,2]-tt),"\n")
#     cat("qq",qq,"taumat[jj,1]",taumat[jj,1],"hv$hattau[1]",hv$hattau[1],"\n")
#     cat("out[jj,]",out[jj,],"hv$what",hv$what,"\n")
   }
   return(out)
}
