#' Slope of secant approximating region in w space that is associated with XV>=xv.
#' @param w1 first SRLLR parameter.
#' @param xvec Bivariate data vector
#' @param K Cumulant generating function
#' @param hv Output from hatwvec associated with xvec and K.
#' @export
#' @return Slope of secant approximating region in w space that is associated with XV>=xv.
zeta2 <- function(w1,xvec,K,hv){
#  cat("Entering zeta2, w1",w1,"\n")
   if(is.na(w1)){
      tau1<-hv$hattau[1]
      w1 <- hv$what[1]
   }else{
      tau1 <- tau1f(w1,xvec,K,hv)
   }
#  cat("In zeta2 w1",w1,"tau1",tau1,"\n")
#  cat("In zeta2 hv$what",hv$what,"\n")
   if(w1==0){
#     cat("In zeta2 zero branch\n")
      temp <- tautilde(0,xvec,K)
#     cat("temp",temp,"hv$what[1]",hv$what[1],"\n")
# Derivative of w1 re tau1 evaluated at zero, from equation w11
      if(hv$what[1]==0){
#        cat("In zeta2 zero first zero branch\n")
         Kz <- K(c(0,temp))$d2
         w11zero <- sqrt(Kz[1,1]-Kz[1,2]*Kz[2,1]/Kz[2,2])
      }else{
#        cat("In zeta2 zero nonzero branch\n")
         w11zero <- (xvec[1]-K(c(0,temp))$d1[1])/hv$what[1] 
      }
# Derivative of w2 re tau1 evaluated at zero, from equation w21
#     cat("w11zero",w11zero,"\n")
      if(hv$what[2]==0){
#        cat("In zeta2 zero second zero branch\n")
         Kz <- K(c(0,0))$d2
         w21zero <- sqrt(Kz[1,1]-Kz[1,2]*Kz[2,1]/Kz[2,2])
      }else{
         w21zero <- ( K(c(0,temp))$d1[1]- K(c(0,0))$d1[1])/hv$what[2] 
      }
      z2 <- w21zero/w11zero
   }else{ 
      w2 <- wvec(c(tau1,0),xvec,hv,K=K)[2]
#     cat("In zeta2 non-zero branch, w1",w1,"w2",w2,"\n")
      z2 <- w2/w1
   }
#  cat("Leaving zeta2, z2",z2,"\n")
   return(z2)
}
