#install.packages("C:\\Users\\Dohy\\Desktop\\Research\\LeeKolassa\\twodsp_1.0.tar.gz",type="source")

#install.packages("C:\\Users\\Dohy\\Desktop\\Research\\LeeKolassa\\VGAM_1.1-8.zip",type="source")

#install.packages("nleqslv")
library(VGAM) 
library(nleqslv) # for twodsp package
library(twodsp)


## Data are assumed to be Canonical cases
## X Y Z have Gamma Distribution with a parameter 1 
## X1 = X + Y
## X2 = X + Z


set.seed(6961402)
## Sampling
nmc<-10000
nsam<-1
z<-rgamma(nmc,nsam)
xv<-cbind(rgamma(nmc,nsam)+z,rgamma(nmc,nsam)+z)/nsam



hatwvec <- function(xvec=NULL,K,hattau=NULL,verbose=FALSE){
  myK<-K
  if(verbose) cat("Entered hatwvec, xvec",xvec,"\n")
  #  cat("Calculate tilde\n")
  if(is.null(hattau)){
    if(verbose) cat("Branch 1\n")
    tildetau <- c(0,tautilde(0,xvec,K=myK))
    hattau <- if(is.na(tildetau[2])) c(NA,NA) else
      try(nleqslv(tildetau,function(tauv,xb) return(K(tauv)$d1-xb),xb=xvec,
                  control=list(xtol=1.0e-12))$x)
    if(verbose) cat("Branch 1b, hattau",hattau,"\n")
    if(inherits(hattau,"try-error")) hattau <- c(NA,NA)
    if(verbose) cat("Finished first optimization, hattau",hattau,"\n")
  }else{
    if(verbose) cat("Branch 2\n")
    xvec<-K(hattau)$d1
    tildetau <- c(0,tautilde(0,xvec,K=myK))
    if(verbose) cat("tildetau",tildetau,"\n")
  }
  if(verbose) cat("Calculated hattau\n")
  big <- xvec%*%hattau -K(hattau)$d0 
  if(verbose) cat("big",big,"\n")
  if(is.na(big)) cat("NA in Big",xvec,"hattau",hattau,"\n")
  small <- xvec%*%tildetau -K(tildetau)$d0 
  if(verbose) cat("small",small,"\n")
  hw1 <- sqrt(2*max(big-small,0))
  if(hattau[1]<0) hw1 <- -hw1
  hw2 <- sqrt(2*max(small,0))
  if(tildetau[2]<0) hw2 <- -hw2
  if(verbose) cat("hw1",hw1,"hw2",hw2,"\n")
  hv <- list(what=c(hw1,hw2),hattau=hattau,tildetau=tildetau,big=big,small=small,xvec=xvec)
  hv$slope <- c(((hattau[2]-tildetau[2])*hw2)/( tildetau[2]*hw1),
                -zeta2(NA,xvec,K,hv))
  ht1z <- c(hattau[1],0)
  Kout <- K(ht1z)
  Kstr <- K(hattau)$d2
  w2z <- wvec(ht1z,xvec,hv,K)[1,2]
  if(verbose) cat("-hv$slope[2]",-hv$slope[2],"w2z/hw1",w2z/hw1,"\n")
  #  cat("first", (Kout$d1[1]-xvec[1])/((w2z-hw2)*hw1*sqrt(Kstr[1,1]-Kstr[2,1]*Kstr[1,2]/Kstr[2,2])),"second",w2z/hw1^2,"\n")
  hv$gp <- (Kout$d1[1]-xvec[1])/((w2z-hw2)*hw1*sqrt(Kstr[1,1]-Kstr[2,1]*Kstr[1,2]/Kstr[2,2])) - (w2z/hw1)/hw1
  #  cat("Leaving hatwvec\n")
  hv$hu <- c(hattau[1]*sqrt(Kstr[1,1]-Kstr[1,2]*Kstr[2,1]/Kstr[2,2]),hattau[2]*sqrt(Kstr[2,2]))
  return(hv)
}


twodsp_star1<-function(xvec=c(4,5),n=1, K=Kdep,stats=NULL,verbose=FALSE){
  #  cat("Entered twodsp, xvec=",xvec,"\n")
  myK<-K
  if(is.null(stats)) stats <- hatwvec(xvec=xvec,K=myK,verbose=verbose)
  if(verbose) cat("n=",n,"\n")
  Khat<-K(stats$hattau)
  Kout<-K(c(0,0))
  if(verbose) cat("Kout$d1",Kout$d1,"\n")
  # Remember zeta2=-slope[2]
  if(verbose) cat("stats$what",stats$what,"\n")
  if(verbose) cat("-stats$slope[2]",-stats$slope[2],"\n")
  stats$whstar<-c(stats$what[1]-log(stats$what[1]/stats$hu[1])/stats$what[1],
                  stats$what[2]-log(stats$what[2]/stats$hu[2])/stats$what[2])
  lrnorm <- if(!any(is.na(stats$hattau)))
    pbinorm(-sqrt(n)*(stats$whstar[1]+stats$whstar[2]*(-stats$slope[2])),
            -sqrt(n)*stats$whstar[2],
            mean1=0,mean2=0,
            var1=1+stats$slope[2]^2, 
            var2=1,cov12=-stats$slope[2])
  else NA
  #  cat("Covariance",-stats$slope[2],"\n")
  if(verbose) cat("stats$hattau",stats$hattau,"\n")
  if(verbose) cat("Normal approximation to bivariate distribution of SRLLR",lrnorm,"\n")
  sigmav<-sqrt(c(Khat$d2[1,1]-Khat$d2[1,2]*Khat$d2[2,1]/Khat$d2[2,2],Khat$d2[2,2]))
  zv<-stats$hattau*sigmav
  hgstv<-c(stats$whstar[1], stats$whstar[2]-(-stats$slope[2])*stats$whstar[1])
  if(verbose) cat("sigmav",sigmav,"hat tau",stats$hattau,"zv",zv,"hgstv",hgstv,"\n")
  dd<-c(dnorm(sqrt(n)*(stats$whstar[1]+stats$whstar[2]*(-stats$slope[2]))/sqrt(1+stats$slope[2]^2))/sqrt(1+stats$slope[2]^2),
        dnorm(sqrt(n)*stats$whstar[2]))
  pp<-c(pnorm(-sqrt(n)*(stats$whstar[2]-(-stats$slope[2])* stats$whstar[1])/sqrt(1+stats$slope[2]^2)),
        pnorm(-sqrt(n)*stats$whstar[1]))
  if(verbose) cat("dd",dd,"pp",pp,"pp*dd",pp*dd,"fractions",(1/zv-1/hgstv),"prods",dd*pp*(1/zv-1/hgstv),"\n")
  rawlr<- if(all(abs(hgstv)>0.0001)) lrnorm+sum(pp*dd*(1/zv-1/hgstv))/sqrt(n) else NA
  if(verbose) cat("Bivariate SRLLR corrected a la LR",rawlr,"\n")
  clt<-pbinorm(-sqrt(n)*(xvec[1]-Kout$d1[1]),-sqrt(n)*(xvec[2]-Kout$d1[2]),mean1=0,mean2=0, 
               var1=Kout$d2[1,1], var2=Kout$d2[2,2], cov12=Kout$d2[1,2])
  integras<-integs(-stats$slope[2],sqrt(n)*stats$whstar)
  if(verbose) print(integras)
  zeta2 <- -stats$slope[2]
  zp <- stats$gp
  vars<-c("omegahat1","omegahat2","zeta2","zeta21")
  vals<-c(stats$whstar,zeta2,zp)
  cat("myrules={",paste(vars,vals,sep="->",collapse=","),"}\n",file="tempa.m")
  vals[4]<-0
  cat("myrules0={",paste(vars,vals,sep="->",collapse=","),"}\n",file="tempa.m",
      append=TRUE)
  if(verbose) cat("zeta2",zeta2,"zeta21",zp,"\n")
  c1 <- as.numeric(zp*integras["k0m11"])
  c2 <- as.numeric(zp*(zeta2+zp*hgstv[1]/2)*integras["k0m20"])
  if(verbose) cat("c1",c1,"c2",c2,"\n")
  if(verbose) cat("zp",zp,"zp*(zeta2+zp*hgstv[1]/2",zp*(zeta2+zp*hgstv[1]/2),"\n")
  corr <- as.numeric(zp*(integras["k0m11"]+(zeta2+zp*hgstv[1]/2)*integras["k0m20"]))/sqrt(n)
  #  browser()
  #  cat("corr",corr,"\n")
  final=rawlr+corr
  if(verbose) cat("Bivariate Saddlepoint including curve",final,"\n")
  return(list(approx=c(clt=clt,lrnorm=lrnorm,rawlr=rawlr,final=final),
              zeta2=zeta2,zeta21=zp,hattau=stats$hattau,hgstv=hgstv,hu=stats$hu))
}

twodsp_star2<-function(xvec=c(4,5),n=1, K=Kdep,stats=NULL,verbose=FALSE){
  #  cat("Entered twodsp, xvec=",xvec,"\n")
  myK<-K
  if(is.null(stats)) stats <- hatwvec(xvec=xvec,K=myK,verbose=verbose)
  if(verbose) cat("n=",n,"\n")
  Khat<-K(stats$hattau)
  Kout<-K(c(0,0))
  if(verbose) cat("Kout$d1",Kout$d1,"\n")
  # Remember zeta2=-slope[2]
  if(verbose) cat("stats$what",stats$what,"\n")
  if(verbose) cat("-stats$slope[2]",-stats$slope[2],"\n")
  hgv<-c(stats$what[1]+stats$what[2]*(-stats$slope[2]),stats$what[2])
  hgstv<-hgv-log(hgv/stats$hu)/hgv
  lrnorm <- if(!any(is.na(stats$hattau)))
    pbinorm(-sqrt(n)*(hgstv[1]),
            -sqrt(n)*(hgstv[2]),
            mean1=0,mean2=0,
            var1=1+stats$slope[2]^2, 
            var2=1,cov12=-stats$slope[2])
  else NA
  #  cat("Covariance",-stats$slope[2],"\n")
  if(verbose) cat("stats$hattau",stats$hattau,"\n")
  if(verbose) cat("Normal approximation to bivariate distribution of SRLLR",lrnorm,"\n")
  sigmav<-sqrt(c(Khat$d2[1,1]-Khat$d2[1,2]*Khat$d2[2,1]/Khat$d2[2,2],Khat$d2[2,2]))
  zv<-stats$hattau*sigmav
  if(verbose) cat("sigmav",sigmav,"hat tau",stats$hattau,"zv",zv,"hgstv",hgstv,"\n")
  dd<-c(dnorm(sqrt(n)*(hgstv[1])/sqrt(1+stats$slope[2]^2))/sqrt(1+stats$slope[2]^2),
        dnorm(sqrt(n)*(hgstv[2])))
  pp<-c(pnorm(sqrt(n)*(hgstv[1])/sqrt(1+stats$slope[2]^2))/sqrt(1+stats$slope[2]^2),
        pnorm(sqrt(n)*(hgstv[2])))
  if(verbose) cat("dd",dd,"pp",pp,"pp*dd",pp*dd,"fractions",(1/zv-1/hgstv),"prods",dd*pp*(1/zv-1/hgstv),"\n")
  rawlr<- if(all(abs(hgstv)>0.0001)) lrnorm+sum(pp*dd*(1/zv-1/hgstv))/sqrt(n) else NA
  if(verbose) cat("Bivariate SRLLR corrected a la LR",rawlr,"\n")
  clt<-pbinorm(-sqrt(n)*(xvec[1]-Kout$d1[1]),-sqrt(n)*(xvec[2]-Kout$d1[2]),mean1=0,mean2=0, 
               var1=Kout$d2[1,1], var2=Kout$d2[2,2], cov12=Kout$d2[1,2])
  integras<-integs(-stats$slope[2],sqrt(n)*hgstv)
  if(verbose) print(integras)
  zeta2 <- -stats$slope[2]
  zp <- stats$gp
  vars<-c("omegahat1","omegahat2","zeta2","zeta21")
  vals<-c(hgstv,zeta2,zp)
  cat("myrules={",paste(vars,vals,sep="->",collapse=","),"}\n",file="tempa.m")
  vals[4]<-0
  cat("myrules0={",paste(vars,vals,sep="->",collapse=","),"}\n",file="tempa.m",
      append=TRUE)
  if(verbose) cat("zeta2",zeta2,"zeta21",zp,"\n")
  c1 <- as.numeric(zp*integras["k0m11"])
  c2 <- as.numeric(zp*(zeta2+zp*hgstv[1]/2)*integras["k0m20"])
  if(verbose) cat("c1",c1,"c2",c2,"\n")
  if(verbose) cat("zp",zp,"zp*(zeta2+zp*hgstv[1]/2",zp*(zeta2+zp*hgstv[1]/2),"\n")
  corr <- as.numeric(zp*(integras["k0m11"]+(zeta2+zp*hgstv[1]/2)*integras["k0m20"]))/sqrt(n)
  #  browser()
  #  cat("corr",corr,"\n")
  final=rawlr+corr
  if(verbose) cat("Bivariate Saddlepoint including curve",final,"\n")
  return(list(approx=c(clt=clt,lrnorm=lrnorm,rawlr=rawlr,final=final),
              zeta2=zeta2,zeta21=zp,hattau=stats$hattau,hgstv=hgstv,hu=stats$hu))
}

uhat1<-apply(xv,1,function(x) hatwvec(xvec=x,K=Kdep,hattau=NULL,verbose=FALSE)$hu[1])
uhat2<-apply(xv,1,function(x) hatwvec(xvec=x,K=Kdep,hattau=NULL,verbose=FALSE)$hu[2])
zeta2<-apply(xv,1,function(x) hatwvec(xvec=x,K=Kdep,hattau=NULL,verbose=FALSE)$slope[2])
what1<-apply(xv,1,function(x) hatwvec(xvec=x,K=Kdep,hattau=NULL,verbose=FALSE)$what[1])
what2<-apply(xv,1,function(x) hatwvec(xvec=x,K=Kdep,hattau=NULL,verbose=FALSE)$what[2])

#wsthat1 <- what1 - log(what1/uhat1)/what1
#wsthat2 <- what2 - log(what2/uhat2)/what2


ghat<-cbind(what1+what2*(-zeta2),what2)
uh<-cbind(uhat1,uhat2)

par(mfrow=c(2,2))
plot(ghat[,1],uh[,1],xlab="gammahat1",ylab="uhat1",
     main="Gamma_hat1 vs u_hat1")
abline(a=0,b=1,col="red")
plot(ghat[,2],uh[,2],xlab="gammahat2",ylab="uhat2",
     main="Gamma_hat2 vs u_hat2")
abline(a=0,b=1,col="red")


## replace lrnorm parts to r* approximation
## lrnorm
n <- 1
ghat.cdf<-pbinorm(-sqrt(n)*ghat[,1],
         -sqrt(n)*ghat[,2],
          mean1=0,mean2=0,
          var1=1+zeta2^2, 
          var2=1,
          cov12=-zeta2)

plot(ghat[,1],ghat[,2],
     xlab="gammahat1",ylab="gammahat2",
     col = ifelse(xv[,1]>mean(xv[,1]) & xv[,2]>mean(xv[,2]),"red","green"))

sample<-rbinorm(10000,mean1=0,mean2=0,
        var1=1+zeta2^2, 
        var2=1,
        cov12=-zeta2)

sample.cdf<-pbinorm(sample[,1],sample[,2],
                  mean1=0,mean2=0,
                  var1=1+zeta2^2, 
                  var2=1,
                  cov12=-zeta2)

plot(sort(sample.cdf),sort(ghat.cdf),col="green",
     xlab="cdf with sample",ylab="cdf with ghatv")
abline(a=0,b=1,col="red")


#library(car)
#scatter3d(-sqrt(n)*ghat[,1],-sqrt(n)*ghat[,2],ghat.cdf,point.col="blue", surface=FALSE)




